/*  
 * Copyright (c) 2010 Ant Kutschera, maxant
 * 
 * This file is part of the maxant Blog.
 * 
 * maxant Blog is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant Blog is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with maxant Blog.  If not, see <http://www.gnu.org/licenses/>.
 */
package uk.co.maxant.wsdemo;

import java.io.IOException;
import java.net.URL;

import javax.xml.XMLConstants;
import javax.xml.transform.Source;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;
import javax.xml.ws.LogicalMessage;
import javax.xml.ws.WebServiceException;
import javax.xml.ws.handler.LogicalHandler;
import javax.xml.ws.handler.LogicalMessageContext;
import javax.xml.ws.handler.MessageContext;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/**
 * a logical handler, used to validate the incoming data against the xsd.
 */
public class MyLogicalHandler implements LogicalHandler<LogicalMessageContext> {

	/** reusable validator instance */
	private Validator validator;

	private boolean inited = false;
	
	/**
	 * would like to do this in the constructor, but it leads to a deadlock because at the time
	 * the handler is instantiated, the web service is still starting up and the XSD isn't online!
	 * you could use an offline XSD...
	 */
	private void init(){
		if(!inited){
			synchronized (this) {
				if(!inited){
					try{
						//instantiate the validator just once
						long start = System.nanoTime();					
						SchemaFactory schemaFac = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
						Schema schema = schemaFac.newSchema(new URL("http://localhost:8084/Gisler/GislerServiceService?xsd=1"));
						validator = schema.newValidator();
						System.out.println("created validator in " + ((System.nanoTime()-start)/1000000.0) + "ms");
						inited = true;
					} catch (IOException e) {
						e.printStackTrace();
						throw new WebServiceException(e.getMessage());  //cant initialise
					} catch (SAXException e) {
						e.printStackTrace();
						throw new WebServiceException(e.getMessage());  //cant initialise
					}
				}
			}
		}
	}
	
	@Override
	public void close(MessageContext paramMessageContext) {
	}
	
	@Override
	public boolean handleFault(LogicalMessageContext paramC) {
		return true;
	}
	
	@Override
	public boolean handleMessage(LogicalMessageContext context) {

		init();
		
		long start = System.nanoTime();
		
        if (((Boolean) context.get(MessageContext.MESSAGE_OUTBOUND_PROPERTY)).booleanValue()) {
            return true; // only validate incoming messages
        }

        String result = "FAILED";
        try{
        	//validate against XSD...
        	LogicalMessage lm = context.getMessage();
        	Source payload = lm.getPayload();
            validator.validate(payload);
            result = "OK";
        }catch(SAXParseException e){
        	System.out.println("validated failed: " + e.getMessage());
        	throw new WebServiceException(e.getMessage());  //invalid, so tell the caller!
        } catch (SAXException e) {
        	e.printStackTrace();
        	throw new WebServiceException(e.getMessage());  //cant validate
		} catch (IOException e) {
        	e.printStackTrace();
        	throw new WebServiceException(e.getMessage());  //cant validate
		}finally{
			System.out.println("validated [" + result + "] in " + ((System.nanoTime()-start)/1000000.0) + "ms");
		}

		return true;
	}
}
